document.addEventListener('DOMContentLoaded', function() {
    // Piano settings
    const NUM_OCTAVES = 3;
    const STARTING_OCTAVE = 3;
    let currentOctave = 2; // Default octave (middle C is C4)
    let currentSound = 'grand-piano'; // Default sound

    // Audio context
    let audioContext;
    
    // Active oscillators
    const activeOscillators = {};

    // Create piano keys
    createPianoKeys();

    // Generate piano note frequencies for multiple octaves
    const noteFrequencies = {};
    
    for (let octave = 1; octave <= 7; octave++) {
        // C in each octave is calculated as 261.63 * 2^(octave-4)
        // Middle C is C4 (octave 4)
        const baseC = 261.63 * Math.pow(2, octave - 4);
        
        noteFrequencies[`C${octave}`] = baseC;
        noteFrequencies[`C#${octave}`] = baseC * Math.pow(2, 1/12);
        noteFrequencies[`Db${octave}`] = baseC * Math.pow(2, 1/12);
        noteFrequencies[`D${octave}`] = baseC * Math.pow(2, 2/12);
        noteFrequencies[`D#${octave}`] = baseC * Math.pow(2, 3/12);
        noteFrequencies[`Eb${octave}`] = baseC * Math.pow(2, 3/12);
        noteFrequencies[`E${octave}`] = baseC * Math.pow(2, 4/12);
        noteFrequencies[`F${octave}`] = baseC * Math.pow(2, 5/12);
        noteFrequencies[`F#${octave}`] = baseC * Math.pow(2, 6/12);
        noteFrequencies[`Gb${octave}`] = baseC * Math.pow(2, 6/12);
        noteFrequencies[`G${octave}`] = baseC * Math.pow(2, 7/12);
        noteFrequencies[`G#${octave}`] = baseC * Math.pow(2, 8/12);
        noteFrequencies[`Ab${octave}`] = baseC * Math.pow(2, 8/12);
        noteFrequencies[`A${octave}`] = baseC * Math.pow(2, 9/12);
        noteFrequencies[`A#${octave}`] = baseC * Math.pow(2, 10/12);
        noteFrequencies[`Bb${octave}`] = baseC * Math.pow(2, 10/12);
        noteFrequencies[`B${octave}`] = baseC * Math.pow(2, 11/12);
    }

    // Keyboard mapping for one octave
    const keyboardMapBase = {
        'a': 'C', 'w': 'C#', 's': 'D', 'e': 'D#', 'd': 'E',
        'f': 'F', 't': 'F#', 'g': 'G', 'y': 'G#', 'h': 'A',
        'u': 'A#', 'j': 'B', 'k': 'C' // 'k' is C of the next octave
    };

    // Function to create piano keys
    function createPianoKeys() {
        const piano = document.getElementById('piano');
        piano.innerHTML = ''; // Clear any existing keys
        
        // Create keys for multiple octaves
        for (let octave = 1; octave <= NUM_OCTAVES; octave++) {
            const octaveOffset = (octave - 1) * 7 * 40; // 7 white keys per octave, 40px per key
            
            // Create white keys
            const whiteKeys = ['C', 'D', 'E', 'F', 'G', 'A', 'B'];
            whiteKeys.forEach((note, index) => {
                const key = document.createElement('div');
                key.className = 'white-key';
                key.dataset.note = note;
                key.dataset.octave = octave + STARTING_OCTAVE - 1;
                key.textContent = note + (octave + STARTING_OCTAVE - 1);
                key.style.left = `${octaveOffset + index * 40}px`;
                piano.appendChild(key);
                
                // Add event listeners
                key.addEventListener('mousedown', () => playNote(note + (octave + STARTING_OCTAVE - 1)));
                key.addEventListener('mouseup', () => stopNote(note + (octave + STARTING_OCTAVE - 1)));
                key.addEventListener('mouseleave', () => stopNote(note + (octave + STARTING_OCTAVE - 1)));
            });
            
            // Create black keys
            const blackKeys = ['C#', 'D#', null, 'F#', 'G#', 'A#', null];
            const blackKeyPositions = [0, 1, 3, 4, 5];
            blackKeyPositions.forEach((position) => {
                const note = blackKeys[position];
                const key = document.createElement('div');
                key.className = 'black-key';
                key.dataset.note = note;
                key.dataset.octave = octave + STARTING_OCTAVE - 1;
                key.textContent = note + (octave + STARTING_OCTAVE - 1);
                
                // Position black keys between white keys
                const whiteKeyWidth = 40;
                const blackKeyOffset = 27;
                key.style.left = `${octaveOffset + position * whiteKeyWidth + blackKeyOffset}px`;
                
                piano.appendChild(key);
                
                // Add event listeners
                key.addEventListener('mousedown', () => playNote(note + (octave + STARTING_OCTAVE - 1)));
                key.addEventListener('mouseup', () => stopNote(note + (octave + STARTING_OCTAVE - 1)));
                key.addEventListener('mouseleave', () => stopNote(note + (octave + STARTING_OCTAVE - 1)));
            });
        }
    }
    
    // Function to initialize audio context
    function initAudioContext() {
        if (!audioContext) {
            audioContext = new (window.AudioContext || window.webkitAudioContext)();
        }
    }
    
    // Function to play a note
    function playNote(noteWithOctave) {
        initAudioContext();
        
        if (activeOscillators[noteWithOctave]) {
            return;
        }
        
        const frequency = noteFrequencies[noteWithOctave];
        if (!frequency) return;
        
        const oscillator = audioContext.createOscillator();
        const gainNode = audioContext.createGain();
        
        // Set oscillator type based on selected sound
        switch(currentSound) {
            case 'electric-piano':
                oscillator.type = 'triangle';
                break;
            case 'organ':
                oscillator.type = 'square';
                break;
            case 'synthesizer':
                oscillator.type = 'sawtooth';
                break;
            case 'grand-piano':
            default:
                oscillator.type = 'sine';
                break;
        }
        
        oscillator.frequency.value = frequency;
        
        gainNode.gain.value = 0.5;
        
        // Add some effects based on sound type
        if (currentSound === 'synthesizer') {
            // Add a filter for synthesizer
            const filter = audioContext.createBiquadFilter();
            filter.type = 'lowpass';
            filter.frequency.value = 1000;
            filter.Q.value = 5;
            
            oscillator.connect(filter);
            filter.connect(gainNode);
        } else {
            oscillator.connect(gainNode);
        }
        
        gainNode.connect(audioContext.destination);
        
        oscillator.start();
        
        activeOscillators[noteWithOctave] = {
            oscillator: oscillator,
            gainNode: gainNode
        };
        
        // Highlight the key
        const note = noteWithOctave.replace(/\d+/, '');
        const octave = noteWithOctave.match(/\d+/)[0];
        const keys = document.querySelectorAll(`[data-note="${note}"][data-octave="${octave}"]`);
        keys.forEach(key => {
            key.classList.add('active');
        });
    }
    
    // Function to stop a note
    function stopNote(noteWithOctave) {
        if (!activeOscillators[noteWithOctave]) {
            return;
        }
        
        const { oscillator, gainNode } = activeOscillators[noteWithOctave];
        
        // Fade out to avoid clicking sounds
        gainNode.gain.setValueAtTime(gainNode.gain.value, audioContext.currentTime);
        gainNode.gain.exponentialRampToValueAtTime(0.001, audioContext.currentTime + 0.1);
        
        setTimeout(() => {
            oscillator.stop();
            delete activeOscillators[noteWithOctave];
        }, 100);
        
        // Remove highlight
        const note = noteWithOctave.replace(/\d+/, '');
        const octave = noteWithOctave.match(/\d+/)[0];
        const keys = document.querySelectorAll(`[data-note="${note}"][data-octave="${octave}"]`);
        keys.forEach(key => {
            key.classList.remove('active');
        });
    }
    
    // Handle keyboard input
    document.addEventListener('keydown', (event) => {
        const key = event.key.toLowerCase();
        
        // Handle octave changes
        if (key === 'z' && !event.repeat) {
            changeOctave(currentOctave - 1);
            return;
        }
        if (key === 'x' && !event.repeat) {
            changeOctave(currentOctave + 1);
            return;
        }
        
        if (keyboardMapBase[key] && !event.repeat) {
            const note = keyboardMapBase[key];
            
            // Handle special case for 'k' key (next octave C)
            const octave = key === 'k' ? currentOctave + 1 : currentOctave;
            
            playNote(note + octave);
        }
    });
    
    document.addEventListener('keyup', (event) => {
        const key = event.key.toLowerCase();
        
        if (keyboardMapBase[key]) {
            const note = keyboardMapBase[key];
            
            // Handle special case for 'k' key (next octave C)
            const octave = key === 'k' ? currentOctave + 1 : currentOctave;
            
            stopNote(note + octave);
        }
    });
    
    // Function to change octave
    function changeOctave(newOctave) {
        if (newOctave >= 1 && newOctave <= 6) {
            currentOctave = newOctave;
            document.getElementById('current-octave').textContent = newOctave;
            
            // Update active octave button
            document.querySelectorAll('.octave-btn').forEach(btn => {
                btn.classList.remove('active');
            });
            
            const activeBtn = document.querySelector(`.octave-btn[data-octave="${newOctave}"]`);
            if (activeBtn) {
                activeBtn.classList.add('active');
            }
        }
    }
    
    // Octave selector buttons
    document.querySelectorAll('.octave-btn').forEach(btn => {
        btn.addEventListener('click', () => {
            const octave = parseInt(btn.dataset.octave);
            changeOctave(octave);
        });
    });
    
    // Sound selector buttons
    document.querySelectorAll('.sound-btn').forEach(btn => {
        btn.addEventListener('click', () => {
            currentSound = btn.dataset.sound;
            
            // Update active sound button
            document.querySelectorAll('.sound-btn').forEach(b => {
                b.classList.remove('active');
            });
            btn.classList.add('active');
        });
    });
    
    // Sheet music player
    let isPlaying = false;
    let currentNoteIndex = 0;
    let noteInterval;
    
    // Function to parse sheet music input
    function parseSheetMusic(input) {
        // Remove extra spaces and split by commas or spaces
        const cleanInput = input.replace(/\s+/g, ' ').trim();
        return cleanInput.split(/[\s,]+/);
    }
    
    // Function to play sheet music
    function playSheetMusic() {
        if (isPlaying) {
            stopSheetMusic();
        }
        
        const sheetMusicText = document.getElementById('sheet-music').value;
        const notes = parseSheetMusic(sheetMusicText);
        
        if (notes.length === 0 || notes[0] === '') {
            return;
        }
        
        isPlaying = true;
        currentNoteIndex = 0;
        
        function playNextNote() {
            if (currentNoteIndex >= notes.length) {
                stopSheetMusic();
                return;
            }
            
            // Stop any currently playing notes
            Object.keys(activeOscillators).forEach(note => {
                stopNote(note);
            });
            
            const note = notes[currentNoteIndex];
            
            // Check if the note has a valid format (e.g., "C4", "Eb5")
            const noteMatch = note.match(/^([A-G][b#]?)(\d+)$/i);
            
            if (noteMatch) {
                const noteName = noteMatch[1];
                const noteOctave = noteMatch[2];
                const noteToPlay = noteName + noteOctave;
                
                // Play the note if it's valid
                if (noteFrequencies[noteToPlay]) {
                    playNote(noteToPlay);
                }
            }
            
            currentNoteIndex++;
        }
        
        playNextNote();
        noteInterval = setInterval(playNextNote, 500); // Play each note for 0.5 seconds
    }
    
    // Function to stop playing sheet music
    function stopSheetMusic() {
        isPlaying = false;
        clearInterval(noteInterval);
        
        // Stop any currently playing notes
        Object.keys(activeOscillators).forEach(note => {
            stopNote(note);
        });
    }
    
    // Event listeners for sheet music controls
    document.getElementById('play-sheet-music').addEventListener('click', playSheetMusic);
    document.getElementById('stop-playing').addEventListener('click', stopSheetMusic);
    document.getElementById('clear-sheet-music').addEventListener('click', () => {
        document.getElementById('sheet-music').value = '';
        stopSheetMusic();
    });
    
    // Initialize with default octave
    changeOctave(currentOctave);
});