let allCoins = [];
let currentChart = null;
let selectedPairs = {}; // Track selected tokens with price data

// DexScreener API Endpoint
const DEXSCREENER_API = "https://api.dexscreener.com/latest/dex/tokens/solana";

// Fetch data from DexScreener
async function fetchData() {
    try {
        const response = await fetch(DEXSCREENER_API);
        const data = await response.json();
        allCoins = data.pairs;
        renderCoins(allCoins);
        updateMainChart(); // Update main chart with fetched data
    } catch (error) {
        console.error("Error fetching data:", error);
    }
}

// Render coins with toggle checkboxes
function renderCoins(coins) {
    const container = document.getElementById("results");
    container.innerHTML = coins.slice(0, 20).map(coin => `
        <div class="coin-card">
            <input type="checkbox" class="coin-toggle" data-pair="${coin.pairAddress}" checked>
            <h3>${coin.baseToken.symbol}</h3>
            <p>Price: $${coin.priceUsd}</p>
            <p class="${coin.priceChange.h24 >= 0 ? 'green' : 'red'} badge">
                24h Change: ${coin.priceChange.h24.toFixed(2)}%
            </p>
            <p>Volume: $${(coin.volume.h24 / 1000).toFixed(1)}K</p>
        </div>
    `).join("");

    // Attach event listeners to checkboxes
    document.querySelectorAll(".coin-toggle").forEach(checkbox => {
        const pairAddress = checkbox.dataset.pair;
        selectedPairs[pairAddress] = true; // Default to selected
        checkbox.addEventListener("change", toggleCoin);
    });

    updateMainChart();
}

// Toggle coin on/off in the main chart
async function toggleCoin(event) {
    const pairAddress = event.target.dataset.pair;
    
    if (event.target.checked) {
        selectedPairs[pairAddress] = true;
    } else {
        delete selectedPairs[pairAddress]; // Remove from tracking
    }

    updateMainChart();
}

// Update the main chart dynamically
async function updateMainChart() {
    const datasets = [];

    for (const pairAddress in selectedPairs) {
        try {
            const response = await fetch(`https://api.dexscreener.com/latest/dex/pairs/solana/${pairAddress}`);
            const data = await response.json();
            const pair = data.pair;

            datasets.push({
                label: `${pair.baseToken.symbol} Price`,
                data: [pair.priceChange.h1, pair.priceChange.h24, pair.priceChange.d7],
                borderColor: "#" + Math.floor(Math.random()*16777215).toString(16), // Random color
                tension: 0.4
            });

        } catch (error) {
            console.error("Error fetching pair data:", error);
        }
    }

    // Destroy previous chart
    if (currentChart) currentChart.destroy();

    // Create new chart with updated dataset
    const ctx = document.getElementById("priceChart").getContext("2d");
    currentChart = new Chart(ctx, {
        type: "line",
        data: {
            labels: ["1H", "24H", "7D"],
            datasets: datasets
        },
        options: {
            responsive: true,
            plugins: {
                legend: { labels: { color: "#fff" } }
            },
            scales: {
                y: { ticks: { color: "#fff" } },
                x: { ticks: { color: "#fff" } }
            }
        }
    });
}

// Search functionality
document.getElementById("searchInput").addEventListener("input", (e) => {
    const term = e.target.value.toLowerCase();
    const filtered = allCoins.filter(coin => 
        coin.baseToken.name.toLowerCase().includes(term) || 
        coin.baseToken.symbol.toLowerCase().includes(term)
    );
    renderCoins(filtered);
});

// Initial load
fetchData();
setInterval(fetchData, 360000); // Refresh every minute
